# -*- coding: utf-8 -*- 
# @Time    : 04/14/2024 
# @Author  : nzooherd 
# @File    : telegram_client.py 
# @Software: Visual Studio Code

from logging import getLogger
from typing import Optional
from telethon.types import Peer, PeerRef
from telethon import Client
from telethon._impl.client.types import buttons as btns

from dao.model import Message
from service.message_service import MessageService
from template import BaseTemplate
from utills.bytes_util import CallbackBytesRequest

logger = getLogger()
class ExtClient(Client):

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

        self.message_service = MessageService()
    
    def replace_button_selfref(self, buttons: Optional[list[btns.Button] | list[list[btns.Button]]], message_id: int):
        new_buttons = []
        if not buttons:
            return new_buttons

        def format_button(button: btns.Button) -> btns.Button:
            request: CallbackBytesRequest = button.request
            if not request:
                return button
            request.message_id = message_id
            return  btns.Callback(button.text, request.to_byte())

        for item in buttons:
            if isinstance(item, list):
                row_buttons = []
                for button in item:
                    if isinstance(button, btns.Callback):
                        row_buttons.append(format_button(button))
                    else:
                        row_buttons.append(button)
                new_buttons.append(row_buttons)
                continue

            if isinstance(item, btns.Callback):
                row_buttons.append(format_button(button))
            else:
                new_buttons.append(item)

        return new_buttons

    async def raw_send_message(self, chat: Peer | PeerRef, text: str, /, **kwargs) -> Message:
        """
        Send a message to the given chat and log the message event.
        """
        await super().send_message(chat, text, **kwargs)

    async def send_message(self,
        chat: Peer | PeerRef,
        /,
        *,
        template: BaseTemplate,
        link_preview: bool = False,
        reply_to: Optional[int] = None,
    ) -> Message:
        """
        Send a message to the given chat and log the message event.
        """
        message = await self.message_service.placeholder_message()
        buttons = self.replace_button_selfref(template.buttons, message.id)
        template.buttons = buttons
        t_message = await super().send_message(chat, **template.template(), 
                                               link_preview=link_preview, reply_to=reply_to, 
                                               buttons=template.buttons)

        await self.message_service.new_message(message, t_message, template)

    async def edit_message(self,
        chat: Peer | PeerRef,
        /,
        *,
        old_message: Message,
        template: BaseTemplate,
        link_preview: bool = False,
        buttons: Optional[list[btns.Button] | list[list[btns.Button]]] = None,
    ) -> Message:
        """
        Edit a message in the given chat and log the edit event.
        """
        message = await self.message_service.placeholder_message()
        buttons = self.replace_button_selfref(template.buttons, message.id)
        template.buttons = buttons
        t_message = await super().edit_message(chat, old_message.message_id, **template.template(), 
                                               link_preview=link_preview, buttons=buttons)
        await self.message_service.edit_message(message, old_message.id, t_message, template)
